function experiment1()
%This function carries out experiment (1) in the paper: a study of the
%convergence rate of the algorithm on a large phylogeny.

traitfile = 'EFNdata/EFN_all_trait.txt';
treefile = 'EFNdata/EFNtree.tre'; 
[T,C] = readEFNdata(traitfile,treefile);


%Find a maximum likelihood estimate of mu_r given a diffuse root.
theta.sigma2 = 1.0; %scale such that the variance at non-root node i is
                    %equal to theta.sigma2*[branch length above i]
theta.sigma2_r = theta.sigma2*treeHeight(T);
theta.mu_r = fminsearch(@(mu_r) - runLikelihood(mu_r,theta,T,C,200),-5);

%%%%%%%%%%%%%%%%%%%%%%
%First part: convergence of algorithm on given data.
Ns = [10:10:100,150:50:500,600:100:1000];
sigma_r_vals = [1/10;1;10]*theta.sigma2_r; %different scales of theta.sigma2_r.

theta2 = theta;
Ls = zeros(length(sigma_r_vals),length(Ns));
LsSimp = zeros(length(sigma_r_vals),length(Ns));

for i = 1:length(sigma_r_vals)
    theta2.sigma2_r = sigma_r_vals(i);
    LsSimp(i,:) = thresholdSimpsonsSet(T,C,theta2,Ns);
    Ls(i,:) = thresholdGaussSet(T,C,theta2,Ns);
end

%BACKUP Ls here.
save ExperimentOutput/EFN_convergenceResults.mat theta Ns Ls LsSimp     %Specify filename in ExperimentOutput directory.


%%%%%%%%%%%%%%%%%%%%%%
%Second part: convergence of the algorithm on data simulated on the tree.
Ns = [10:10:100,150:50:500,600:100:1000];
nreps = 20;
theta2 = theta;

L_Simulated = zeros(nreps,length(Ns));
L_SimulatedSimp = zeros(nreps,length(Ns));
for i=1:nreps
    c = simulateTrait(T,theta2);
    L_SimulatedSimp(i,:) = thresholdSimpsonsSet(T,c,theta2,Ns);
    L_Simulated(i,:) = thresholdGaussSet(T,c,theta2,Ns);
end

save ExperimentOutput/EFN_SimulatedResults.mat theta Ns L_Simulated L_SimulatedSimp


%%%%%%%%%%%%%%%%%%%%%%
%Third part: convergence of the algorithm on data simulated random data.
Ns = [10:10:100,150:50:500,600:100:1000];
nreps = 20;
theta2 = theta;

L_CoinFlip = zeros(nreps,length(Ns));
L_CoinFlipSimp = zeros(nreps,length(Ns));
for i=1:nreps
    c = floor(2*rand(T.ntax,1)); %Generate random character
    L_CoinFlipSimp(i,:) = thresholdSimpsonsSet(T,c,theta2,Ns);
    L_CoinFlip(i,:) = thresholdGaussSet(T,c,theta2,Ns);
end

save ExperimentOutput/EFN_CoinFlipResults.mat theta Ns L_CoinFlip L_CoinFlipSimp

plotFigures(Ns,Ls,LsSimp,L_Simulated,L_CoinFlip,1,1)

end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Ls = thresholdSimpsonsSet(T,C,theta,Ns)
    %Approximates the likelihood of C given theta and T for the different
    %values of N (number of segments per interval) with array Ns using the
    %Simpson's rule.
    Ls = zeros(1,length(Ns));
    parfor i = 1:length(Ns)
        Ls(i) = thresholdSimpsons(T,C,theta,Ns(i));
    end
end

function Ls = thresholdGaussSet(T,C,theta,Ns)
    %Approximates the likelihood of C given theta and T for the different
    %values of N (number of segments per interval) with array Ns using the
    %Gaussian kernel method.
    Ls = zeros(1,length(Ns));
    parfor i = 1:length(Ns)
        Ls(i) = thresholdGauss(T,C,theta,Ns(i));
    end
end


function logL = runLikelihood(mu_r,theta,T,C,N)
    %function for finding maximum likelihood estimate of mu_r given
    %theta.sigma2, theta.sigma2_r, T, C, and N.
    theta.mu_r = mu_r;
    logL = thresholdSimpsons(T,C,theta,N);
end


function height = treeHeight(T)
    %Compute height of the tree (assumed ultrametric)
    x = 1;
    height = 0;
    ntax = size(T.labels,1);
    while x<2*ntax-1
        height = height + T.t(x);
        x = T.parent(x);
    end
end


function c = simulateTrait(T,theta)
    %Simulate a trait on the tree under the threshold model.
    X = zeros(T.nnodes,1);
    n = T.ntax;
    X(T.nnodes) = theta.mu_r + sqrt(theta.sigma2_r)*randn();
    for i=(T.nnodes-1):-1:1
        j = T.parent(i);
        X(i) = X(j) + sqrt(theta.sigma2*T.t(i))*randn();
    end
    c = zeros(n,1);
    c(X(1:n)>0) = 1;
end



