function logL = thresholdLikelihood(phyT,C,theta,N)
% logL = thresholdGauss(phyT,C,theta,N)
%
% Compute (approximate) log Likelihood of the characters in C on the tree
% phyT under Wright & Felsensteins threshold model. This algorithm uses a
% Gaussian kernel method. For details see:
%       Hiscott, G., Fox, C. Parry, M. and Bryant, D. Efficient recycled 
%       algorithms for quantitative trait models on phylogenies. 
%       http://arxiv.org/abs/1512.00890
%
% phyT is a Phytree object. It is assumed that the tree is binary: every 
%    internal node of the Phytree has exactly two children.
%
% C is an nleaves x nchars array. This is coding 0-1 characters, with
%   negatives denoting missing data.
%
% theta is a bundle of parameters with the following fields
%   theta.mu_r   mean of root density
%   theta.sigma2_r   variance of root density
%   theta.sigma2    variance of transition densities (per unit branch
%       length)
% If theta.sigma2_r = 0, we assume that the liability at the root is fixed
% at theta.mu_r
%
% N is the number of segments per interval (N >0)
%
% Code copyright Gordon Hiscott, Colin Fox, Matthew Parry and David Bryant
% 2016.
%
% For queries, contact david.bryant@otago.ac.nz

T = convertPhytree(phyT); %Convert the phytree into our own format.
nnodes = size(T.children,1);
nchar = size(C,2);

n = (nnodes+1)/2;
x = (0:N)/N; %array used to calculate the points where the functions are evaluated
p = zeros(nnodes,1); %p(i) = length of path from root to node i
for i = (nnodes-1):-1:1
    p(i) = p(T.parent(i)) + T.t(i);
end
L = zeros(nnodes,1); %lower integration limits for each node's integral
U = zeros(nnodes,1); %upper integration limits for each node's integral
X = zeros(nnodes,N+1);

alpha = n*N^(-4)/(2*(n-1)); %truncation error of each individual integral.
for i=(n+1):nnodes
    L(i) = InvPhi(alpha,theta.mu_r,theta.sigma2_r+theta.sigma2*p(i));
    U(i) = InvPhi(1-alpha,theta.mu_r,theta.sigma2_r+theta.sigma2*p(i));
    X(i,:) = (U(i) - L(i))*x+L(i);
end

logL = 0.0;  %log of the product of the likelihoods
like = zeros(nchar,1); %like(c) = likelihood of the characters in C(c,:)

for ch = 1:nchar
    z = C(:,ch); %trait values observed at the leaves for character ch
    F = ones(nnodes,N+1); %F(i,x) = likelihood of the data at or directly
                          %below node i, given a liability of x at i's
                          %parent node.
    for i=1:n
        j = T.parent(i); %parent node of i.
        for k=0:N
            if z(i) == 1
                %integrates over positive liabilities
                F(i,k+1) = 1 - Phi(0,X(j,k+1),theta.sigma2*T.t(i));
            elseif z(i) == 0
                %integrates over negative liabilities
                F(i,k+1) = Phi(0,X(j,k+1),theta.sigma2*T.t(i));
            end
        end
    end
    for i=(n+1):(2*n-2)
        j = T.parent(i);
        u = T.children(i,1); v = T.children(i,2); %child nodes of node i.
        if T.t(i) == 0
            F(i,:) = F(u,:).*F(v,:);
        else
            %Integral calculated via Gaussian kernel method
            weight = findWeightsSet(X(i,:),X(j,:),theta.sigma2*T.t(i));
            for k=0:N
                f = F(u,:).*F(v,:);
                F(i,k+1) = sum(weight(k+1,:).*f);
            end
        end
    end
    r = 2*n-1;
    u = T.children(r,1); v = T.children(r,2);
    if theta.sigma2_r > 0
        %Integral calculated via Gaussian kernel method
        weight = findWeights(X(r,:),theta.mu_r,theta.sigma2_r);
        f = F(u,:).*F(v,:);
        like(ch)= sum(weight.*f);
    else
        %We have L_r = U_r, so X_r[k] is constant in k. 
        like(ch) = F(u,1)*F(v,1);
    end
    
    if like(ch)>0
        logL = logL + log(like(ch));
    else
        logL = -Inf;
    end
end
end


function [T] = convertPhytree(phyT)
%Convert Phytree format.
% T contains the following fields:
%   T.children  nnodes x 2 array specifying children of each node (or 0 if
%       none)
%   T.parent nnodes x 1 array specifying parents of each node (or 0 if
%       root)
%   T.t  nnodes x 1 array specifying branch lengths. (0 if root)
% The procedure readTree can set up this structure from a NEWICK formatted
% string.

m = get(phyT,'NumNodes');
n = get(phyT,'NumLeaves');
[M,ID,T.t] = getmatrix(phyT);

T.parent = zeros(m,1);
T.children = zeros(m,2);
T.labels = ID(1:n);

[row,col] = find(M); 
for i=1:2*(m-n)
    p = row(i);
    c = col(i);
    T.parent(c) = p;
    if T.children(p,1)==0
        T.children(p,1) = c;
    else
        T.children(p,2) = c;
    end
end

end



function w = findWeights(X,mu,sig2)
    %weights calculated for approximating \int phi(x,mu,sig2) f(x) dx
    N = length(X)-1;
    w = zeros(size(X)); %weight array
    h = X(2) - X(1); %spacing between two consecutive points.
    f = zeros(3,(N/2)+1); %matrix of antiderivatives used to calculate the weights.
    sig = sqrt(sig2);
    %f(1,i) = antiderivative of phi(x,mu,sig2) at x = X(2*i - 1)
    f(1,:) = .5*erf((X(1:2:(N+1))-mu)/(sqrt(2*sig2)));
    %f(2,i) = antiderivative of x*phi(x,mu,sig2) at x = X(2*i - 1)
    f(2,:) = -sig2*phi(X(1:2:(N+1)),mu,sig2) + mu*f(1,:);
    %f(3,i) = antiderivative of x^2*phi(x,mu,sig2) at x = X(2*i - 1)
    f(3,:) = (-2*sig*((X(1:2:(N+1))+mu).*exp(-((X(1:2:(N+1))-mu).^2)/(2*sig2))) + 2*sqrt(2*pi)*(mu^2 + sig2)*f(1,:))/(2*sqrt(2*pi));
    x = X(1:2:N);
    x2 = x.^2;
    f2 = f(:,2:end) - f(:,1:end-1);
    w(1:2:end) = (.5*(((2*h^2)+3*h*[x,0]+[x2,0]).*[f2(1,:),0] + ([0,x].*([0,x+h])).*[0,f2(1,:)]) - .5*(((3*h)+(2*[x,0])).*[f2(2,:),0] + ((2*[0,x])+h).*[0,f2(2,:)]) + .5*([f2(3,:),0] + [0,f2(3,:)]))/(h^2);
    w(2:2:end) = (-((x2+2*h*x).*f2(1,:))+(2*(x+h).*f2(2,:))-f2(3,:))/(h^2);
end


function w = findWeightsSet(X,mu,sig2)
    %a version of findWeights that find weight arrays for multiple values
    %of mu.
    N = length(X)-1;
    w = zeros(N+1); %weight matrix
    h = X(2) - X(1); %spacing between two consecutive points of X.
    sig = sqrt(sig2);
    x = repmat(X(1:2:(N+1)),N+1,1);
    MU = repmat(mu',1,(N/2)+1);
    %f1 = antiderivative of phi(x,mu,sig2) at x = X(2*i - 1)
    f1 = .5*erf((x-MU)/(sqrt(2*sig2)));
    %f2 = antiderivative of x*phi(x,mu,sig2) at x = X(2*i - 1)
    f2 = -sig2*phi(x,MU,sig2) + MU.*f1;
    %f3 = antiderivative of x^2*phi(x,mu,sig2) at x = X(2*i - 1)
    f3 = (-2*sig*((x+MU).*exp(-((x-MU).^2)/(2*sig2))) + 2*sqrt(2*pi)*(MU.^2 + sig2).*f1)/(2*sqrt(2*pi));
    x = x(:,1:(end-1));
    x2 = x.^2;
    F1 = f1(:,2:end) - f1(:,1:end-1);
    F2 = f2(:,2:end) - f2(:,1:end-1);
    F3 = f3(:,2:end) - f3(:,1:end-1);
    z = zeros(N+1,1);
    w(:,1:2:end) = (.5*(((2*h^2)+3*h*[x,z]+[x2,z]).*[F1,z] + ([z,x].*([z,x+h])).*[z,F1]) - .5*(((3*h)+(2*[x,z])).*[F2,z] + ((2*[z,x])+h).*[z,F2]) + .5*([F3,z] + [z,F3]))/(h^2);
    w(:,2:2:end) = (-((x2+2*h*x).*F1)+(2*(x+h).*F2)-F3)/(h^2);
end


function y = phi(x,mu,sigma2)
%normal pdf
y = 1/sqrt(2*pi*sigma2) * exp(-(x - mu).^2/(2*sigma2));
end

function y = Phi(x,mu,sigma2)
%normal cdf
y = 1/2 * (1 + erf((x-mu)/sqrt(2*sigma2)));
end

function y = InvPhi(x,mu,sigma2)
%inverse error function
y = sqrt(2*sigma2)*erfinv(2*x-1)+mu;
end






