function logL = thresholdSimpsons(T,C,theta,N)
% logL = thresholdSimpsons(T,C,theta)
%
% Compute (approximate) log Likelihood of the characters in C on the tree
% T under Wright & Felsensteins threshold model. This algorithm uses the
% Simpson's rule.
%
% T contains the following fields:
%   T.children  nnodes x 2 array specifying children of each node (or 0 if
%       none)
%   T.parent nnodes x 1 array specifying parents of each node (or 0 if
%       root)
%   T.t  nnodes x 1 array specifying branch lengths. (0 if root)
%
% C is an nleaves x nchars array. This is coding 0-1 characters, with
%   negatives denoting missing data.
%
% theta is a bundle of parameters with the following fields
%   theta.mu_r   mean of root density
%   theta.sigma2_r   variance of root density
%   theta.sigma2    variance of transition densities (per unit branch
%       length)
% If theta.sigma2_r = 0, we assume that the liability at the root is fixed
% at theta.mu_r
%
% N is the number of segments per interval (N >0)
%
% Copyright Gordon Hiscott, Colin Fox, Matthew Parry and David Bryant 2016.
%

nnodes = size(T.children,1);
nchar = size(C,2);

n = (nnodes+1)/2; %number of leaves
w = ones(1,N+1); w(2:2:N)=4; w(3:2:N)=2; w = w/3; %w = [1 2 4 2 4 ... 2 1]/3
p = zeros(nnodes,1); %p(i) = length of path from root to node i
for i = (nnodes-1):-1:1
    p(i) = p(T.parent(i)) + T.t(i);
end
L = zeros(nnodes,1); %lower integration limits for each node's integral
U = zeros(nnodes,1); %upper integration limits for each node's integral
X = zeros(nnodes,N+1);

alpha = n*N^(-4)/(2*(n-1)); %truncation error of each individual integral.
for i=(n+1):nnodes
    L(i) = InvPhi(alpha,theta.mu_r,theta.sigma2_r+theta.sigma2*p(i));
    U(i) = InvPhi(1-alpha,theta.mu_r,theta.sigma2_r+theta.sigma2*p(i));
    X(i,:) = linspace(L(i),U(i),N+1);
end

logL = 0.0; %log of the product of the likelihoods
like = zeros(nchar,1); %like(c) = likelihood of the characters in C(c,:)

for ch = 1:nchar
    z = C(:,ch); %trait values observed at the leaves for character ch
    F = ones(nnodes,N+1); %F(i,x) = likelihood of the data at or directly
                          %below node i, given a liability of x at i's
                          %parent node.
    for i=1:n
        j = T.parent(i); %parent node of i.
        for k=0:N
            if z(i) == 1
                %integrates over positive liabilities
                F(i,k+1) = 1 - Phi(0,X(j,k+1),theta.sigma2*T.t(i));
            elseif z(i) == 0
                %integrates over negative liabilities
                F(i,k+1) = Phi(0,X(j,k+1),theta.sigma2*T.t(i));
            end
        end
    end
    for i=(n+1):(2*n-2)
        j = T.parent(i);
        u = T.children(i,1); v = T.children(i,2); %child nodes of node i.
        if T.t(i) == 0
            F(i,:) = F(u,:).*F(v,:);
        else
            for k=0:N
                %integral calculated via Simpson's rule
                f = phi(X(i,:),X(j,k+1),theta.sigma2*T.t(i)).*F(u,:).*F(v,:);
                F(i,k+1) = (U(i)-L(i))/N * sum(w.*f);
            end
        end
    end
    r = 2*n-1;
    u = T.children(r,1); v = T.children(r,2);
    if theta.sigma2_r > 0
        %integral calculated via Simpson's rule
        f = phi(X(r,:),theta.mu_r,theta.sigma2_r).*F(u,:).*F(v,:);
        like(ch)= ((U(r)-L(r))/N * sum(w.*f));
    else
        %We have L_r = U_r, so X_r[k] is constant in k. 
        like(ch) = F(u,1)*F(v,1);
    end
    
    
    if like(ch)>0
        logL = logL + log(like(ch));
    else
        logL = -Inf;
    end
end

end

function y = phi(x,mu,sigma2)
%normal pdf
y = 1/sqrt(2*pi*sigma2) * exp(-(x - mu).^2/(2*sigma2)   );
end

function y = Phi(x,mu,sigma2)
%normal cdf
y = 1/2 * (1 + erf((x-mu)/sqrt(2*sigma2)));
end

function y = InvPhi(x,mu,sigma2)
%inverse error function
y = sqrt(2*sigma2)*erfinv(2*x-1)+mu;
end






